/*
 * (c) Copyright 2004 Mauro Tortonesi - ALL RIGHTS RESERVED
 * 
 * This file is part of the JSR82ext stack.
 * 
 * The JSR82ext stack is free software; you can redistribute it 
 * and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * The JSR82ext stack is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 */

package org.javabluetooth.demo;

import java.io.*;
import org.javabluetooth.stack.l2cap.ASBChannel;


public class ASBxfer implements Runnable {
    private InputStream lin;
    private ASBChannel rem;
    
    ASBxfer(InputStream is, ASBChannel conn) {
        lin = is;
        rem = conn;
    }

    public void run() {
        boolean exit = false;
        byte[] l2rbuf = new byte[672];
        int l2rcc = 0;
        
        while (exit == false) {
            int tmp;
            int avail = 0;
           
            //System.out.println("in the loop");

            try {
                avail = lin.available();
            }
            catch (IOException e) {
                System.err.println("Unexpected I/O failure: " + e);
                System.exit(21);
            }
            
            /* read from local input */
            if (avail > 0 && l2rcc < l2rbuf.length) {
                int cc = 0;
                
                if (l2rcc != 0) {
                    System.err.println("Internal error: l2rcc != 0 while " + 
                                       "reading from stdin.");
                    System.exit(22);
                }
                
                try {
                    cc = lin.read(l2rbuf, l2rcc, l2rbuf.length - l2rcc);
                }
                catch (IOException e) {
                    System.err.println("Unexpected I/O failure reading from " + 
                                       "local input: " + e);
                    System.exit(23);
                }
                
                if (cc < 0) {
                    // EOF on local input triggers program exit 
                    System.err.println("Received EOF from local input: exiting.");
                    exit = true;
                    break;
                } else {
                    l2rcc += cc;
                }
            }
            
            /* write to remote */
            if (l2rcc > 0) {
                /* mauro: java sucks *VERY* badly... */
                byte[] tmpbuf = new byte[l2rcc];
                System.arraycopy(l2rbuf, 0, tmpbuf, 0, l2rcc);
                try {
                    rem.send(tmpbuf);
                    l2rcc = 0;
                }
                catch (IOException e) {
                    System.err.println("Error writing to remote device: " + e);
                    System.exit(24);
                }
            }
        }
    }
}

