/*
 * (c) Copyright 2004 Mauro Tortonesi - ALL RIGHTS RESERVED
 * 
 * This file is part of the JSR82ext stack.
 * 
 * The JSR82ext stack is free software; you can redistribute it 
 * and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * The JSR82ext stack is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 */

package org.javabluetooth.demo;

import java.io.*;
import java.util.LinkedList;
import javax.bluetooth.BluetoothStateException;
import org.javabluetooth.stack.hci.BluezTransport;
import org.javabluetooth.stack.hci.HCIDriver;
import org.javabluetooth.stack.BluetoothStackLocal;
import org.javabluetooth.stack.BluetoothStack;
import org.javabluetooth.stack.sco.BluezSCOConnectionFactory;
import org.javabluetooth.stack.l2cap.ASBChannel;
import org.javabluetooth.util.BluetoothAddress;
import org.javabluetooth.util.BluetoothAddressException;
import org.javabluetooth.util.Debug;
/*
import javax.bluetooth.DataElement;
import javax.bluetooth.DeviceClass;
import javax.bluetooth.DiscoveryAgent;
import javax.bluetooth.DiscoveryListener;
import javax.bluetooth.LocalDevice;
import javax.bluetooth.RemoteDevice;
import javax.bluetooth.ServiceRecord;
import javax.bluetooth.L2CAPConnection;
import javax.microedition.io.Connector;
import org.javabluetooth.stack.hci.BluezTransport;
import org.javabluetooth.stack.hci.HCIDriver;
import org.javabluetooth.stack.BluetoothStackLocal;
import org.javabluetooth.stack.BluetoothStack;
import org.javabluetooth.stack.sco.BluezSCOConnectionFactory;
import org.javabluetooth.util.BluetoothAddress;
import org.javabluetooth.util.BluetoothAddressException;
import org.javabluetooth.util.Debug;
import org.javabluetooth.util.Semaphore;
*/

public class L2CAPBroadcast {

    public static final void main(String[] args) throws Exception {
        short psm = 0;
        FileInputStream file = null;
        
        if (args.length != 2) {
            System.err.println("Error: 2 parameters expected!");
            System.exit(1);
        }

        try {
            file = new FileInputStream(args[0]);
        }
        catch (FileNotFoundException e) {
            System.err.println("Error: file " + args[0] + " not found! " + e);
            System.exit(2);
        }

        try {
            psm = Short.parseShort(args[1], 10);
        }
        catch (NumberFormatException e) {
            System.err.println("Error: invalid PSM value!");
            System.exit(3);
        }

        /* ------------------------------------------------------------
         *                   LOW LEVEL INITIALIZATION
         * ------------------------------------------------------------ */
        /* create HCI driver */
        BluezTransport transport = new BluezTransport("hci0");
        BluezSCOConnectionFactory factory = new BluezSCOConnectionFactory();
        HCIDriver driver = new HCIDriver(transport,factory);
        /* setup the Bluetooth manager */
        BluetoothStack.init(new BluetoothStackLocal(driver));
        /* now we can use BluetoothStack */
        BluetoothStack bluetooth = BluetoothStack.getBluetoothStack();
        bluetooth.send_HCI_Write_Num_Broadcast_Retransmissions((byte)0x00);
        System.out.println("Retransmissions: " + bluetooth.send_HCI_Read_Num_Broadcast_Retransmissions());
        /* ------------------------------------------------------------
         *                END OF LOW LEVEL INITIALIZATION
         * ------------------------------------------------------------ */
        
        byte[] l2rbuf = new byte[150];
        int l2rcc = 0;
        /*
        byte[] r2lbuf = new byte[4096];
        int r2lcc = 0;
        */
        boolean exit = false;
        
        System.out.println("Using PSM = " + psm);
        ASBChannel rem = new ASBChannel(psm);
        
        /*
        BufferedReader lin = new BufferedReader(new InputStreamReader(System.in));
        BufferedWriter lout = new BufferedWriter(new OutputStreamWriter(System.out));
        */
        BufferedInputStream lin = new BufferedInputStream(file);
        //BufferedOutputStream lout = new BufferedOutputStream(System.out);

        //exit = true;
        while (exit == false) {
            int tmp;
           
            //System.out.println("in the loop");

            /* read from local input */
            if (lin.available() > 0 && l2rcc < l2rbuf.length) {
                if (l2rcc != 0) {
                    System.err.println("Internal error: l2rcc != 0 while reading from stdin.");
                    System.exit(1);
                }
                int cc = lin.read(l2rbuf, l2rcc, l2rbuf.length - l2rcc);
                if (cc < 0) {
                    // EOF on local input triggers program exit 
                    System.err.println("Received EOF from local input: exiting.");
                    exit = true;
                    break;
                } else {
                    l2rcc += cc;
                }
            }
            
            //System.out.println("in the loop 2");
            
            /* write to stdout */
            /*
            if (r2lcc > 0) {
                System.out.write(r2lbuf, 0, r2lcc);
                r2lcc = 0;
            }
            */

            //System.out.println("in the loop 3");
            
            /* read from remote */
            /*
            if (rem.ready()) {
                if (r2lcc != 0) {
                    System.err.println("Internal error: r2lcc != 0 while reading from remote.");
                    System.exit(1);
                }
                int cc = rem.receive(r2lbuf);
                if (cc < 0) {
                    // EOF on remote triggers program exit 
                    System.err.println("Received EOF from remote: exiting.");
                    exit = true;
                } else {
                    r2lcc += cc;
                }
            }
            */
            
            //System.out.println("in the loop 4");
            
            /* write to remote */
            if (l2rcc > 0) {
                /* mauro: java sucks *VERY* badly... */
                byte[] tmpbuf = new byte[l2rcc];
                System.arraycopy(l2rbuf, 0, tmpbuf, 0, l2rcc);
                try {
                    rem.send(tmpbuf);
                    l2rcc = 0;
                }
                catch (IOException e) {
                    System.err.println("Error writing to remote device: " + e);
                    System.exit(1);
                }
            }
        }
        
        /*
        String teststr = null;
        teststr = "test 1\n";
        rem.send(teststr.getBytes());
        teststr = "test 2\n";
        rem.send(teststr.getBytes());
        teststr = "test 3\n";
        rem.send(teststr.getBytes());
        */
    }
}
/*
 * vim: et ts=4 sw=4
 */

