/*
 * (c) Copyright 2003 Christian Lorenz - ALL RIGHTS RESERVED
 * (c) Copyright 2004 Mauro Tortonesi - ALL RIGHTS RESERVED
 * 
 * This file is part of the JavaBluetooth Stack.
 * 
 * The JavaBluetooth Stack is free software; you can redistribute it 
 * and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * The JavaBluetooth Stack is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 */

package org.javabluetooth.demo;

import java.io.*;
import java.util.LinkedList;
import javax.bluetooth.BluetoothStateException;
import javax.bluetooth.DataElement;
import javax.bluetooth.DeviceClass;
import javax.bluetooth.DiscoveryAgent;
import javax.bluetooth.DiscoveryListener;
import javax.bluetooth.LocalDevice;
import javax.bluetooth.RemoteDevice;
import javax.bluetooth.ServiceRecord;
import javax.bluetooth.L2CAPConnection;
import javax.microedition.io.Connector;
import org.javabluetooth.stack.hci.BluezTransport;
import org.javabluetooth.stack.hci.HCIDriver;
import org.javabluetooth.stack.BluetoothStackLocal;
import org.javabluetooth.stack.BluetoothStack;
import org.javabluetooth.stack.sco.BluezSCOConnectionFactory;
import org.javabluetooth.util.BluetoothAddress;
import org.javabluetooth.util.BluetoothAddressException;
import org.javabluetooth.util.Debug;
import org.javabluetooth.util.Semaphore;
//import org.javabluetooth.stack.sdp.SDPClientChannel;

/** @author Christian Lorenz, Mauro Tortonesi */
public class L2CAPConnect {
    /*
    private LinkedList remoteDevices;
    private Object mutex;
    //private static ServiceRecord[] serviceRecords;
    private static int INQUIRY_TIMEOUT = 60000;
    */

    public static final void main(String[] args) throws Exception {
        short psm = 0;
        
        if (args.length != 2) {
            System.err.println("Error: 2 parameters expected!");
            System.exit(1);
        }

        boolean b1 = BluetoothAddress.isValidStandardRepr(args[0]);
        boolean b2 = BluetoothAddress.isValidJSR82Repr(args[0]);
        
        Debug.println(1, "b1 is: " + b1);
        Debug.println(1, "b2 is: " + b2);
        
        if (!b1 && !b2) {
            System.err.println("Error: invalid Bluetooth address!");
            System.exit(2);
        }

        try {
            psm = Short.parseShort(args[1], 16);
        }
        catch (NumberFormatException e) {
            System.err.println("Error: invalid PSM value!");
            System.exit(3);
        }

        String jsr82address = b2 ? args[0] : BluetoothAddress.StandardToJSR82Repr(args[0]);
        
        /* ------------------------------------------------------------
         *                   LOW LEVEL INITIALIZATION
         * ------------------------------------------------------------ */
        /* create HCI driver */
        BluezTransport transport = new BluezTransport("hci0");
        BluezSCOConnectionFactory factory = new BluezSCOConnectionFactory();
        HCIDriver driver = new HCIDriver(transport,factory);
        /* setup the Bluetooth manager */
        BluetoothStack.init(new BluetoothStackLocal(driver));
        /* now we can use BluetoothStack */
        BluetoothStack bluetooth = BluetoothStack.getBluetoothStack();
        /* ------------------------------------------------------------
         *                END OF LOW LEVEL INITIALIZATION
         * ------------------------------------------------------------ */
        
        /* create the L2CAPConnect instance */
        L2CAPConnect cn = new L2CAPConnect();
        
        /*
        cn.startInquiry();
        System.out.println(cn.foundDevices() + " devices found.");
        */
        
        L2CAPConnection rem = (L2CAPConnection)Connector.open("btl2cap://" + jsr82address + ":" + args[1]);
        
        byte[] l2rbuf = new byte[4096];
        int l2rcc = 0;
        byte[] r2lbuf = new byte[4096];
        int r2lcc = 0;
        boolean exit = false;

        /*
        BufferedReader lin = new BufferedReader(new InputStreamReader(System.in));
        BufferedWriter lout = new BufferedWriter(new OutputStreamWriter(System.out));
        */
        BufferedInputStream lin = new BufferedInputStream(System.in);
        BufferedOutputStream lout = new BufferedOutputStream(System.out);

        while (exit == false) {
            int tmp;
           
            //System.out.println("in the loop");

            /* read from stdin */
            if (lin.available() > 0 && l2rcc < l2rbuf.length) {
                if (l2rcc != 0) {
                    System.err.println("Internal error: l2rcc != 0 while reading from stdin.");
                    System.exit(1);
                }
                int cc = lin.read(l2rbuf, l2rcc, l2rbuf.length - l2rcc);
                if (cc < 0) {
                    /* EOF on stdin triggers program exit */
                    System.err.println("Received EOF from stdin: exiting.");
                    exit = true;
                    break;
                } else {
                    l2rcc += cc;
                }
            }
            
            //System.out.println("in the loop 2");
            
            /* write to stdout */
            if (r2lcc > 0) {
                System.out.write(r2lbuf, 0, r2lcc);
                r2lcc = 0;
            }

            //System.out.println("in the loop 3");
            
            /* read from remote */
            if (rem.ready()) {
                if (r2lcc != 0) {
                    System.err.println("Internal error: r2lcc != 0 while reading from remote.");
                    System.exit(1);
                }
                int cc = rem.receive(r2lbuf);
                if (cc < 0) {
                    /* EOF on remote triggers program exit */
                    System.err.println("Received EOF from remote: exiting.");
                    exit = true;
                } else {
                    r2lcc += cc;
                }
            }
            
            //System.out.println("in the loop 4");
            
            /* write to remote */
            if (l2rcc > 0) {
                /* mauro: java sucks *VERY* badly... */
                byte [] tmpbuf = new byte[l2rcc];
                System.arraycopy(l2rbuf, 0, tmpbuf, 0, l2rcc);
                try {
                    rem.send(tmpbuf);
                    l2rcc = 0;
                }
                catch (IOException e) {
                    System.err.println("Error writing to remote device: " + e);
                    System.exit(1);
                }
            }
        }
        /*
        String teststr = "test\n";
        conn.send(teststr.getBytes());
        */
        
	    /*
        if (remoteDevice != null) { 
            System.out.println("BDADDR: " + remoteDevice.getBluetoothAddress());
            System.out.println("Name: " + remoteDevice.getFriendlyName(false));
            System.out.println("Major dev class: " + remoteDevice.deviceClass.getMajorDeviceClass());
            System.out.println("Minor dev class: " + remoteDevice.deviceClass.getMinorDeviceClass());
            System.out.println("Service classes: " + remoteDevice.deviceClass.getServiceClasses());
	        */
	        /*
            SDPClientChannel sdpChannel = new SDPClientChannel(remoteDevice, blue);
            bluetooth.connectL2CAPChannel(sdpChannel, remoteDevice, (short)0x0001);
            byte[] uuidList = { 0x35, 0x03, 0x19, 0x10, 0x02 };
            DataElement uuidListElement = new DataElement(uuidList);
            sdpChannel.send_SDP_ServiceSearchRequest((short)1, (short)14, uuidListElement);
	        */
	    /*
        } else {
            System.out.println("remoteDevice == null");
        }
	    */
	    /*
        while (serviceRecords == null) { Thread.sleep(1000); }
        for (int i = 0; i < serviceRecords.length; i++) {
            System.out.println("    " + serviceRecords[i]);
            int[] attrIDs = { 0xff00 }; // reversed the values for range... 
                                        // this avoids ranges starting with 00 
                                        // to be tructated.
            try { 
                serviceRecords[i].populateRecord(attrIDs); 
            }
            catch (IOException e) { 
                System.out.print("ServiceRecord.populateRecord(attrIDs) failed: " + e); 
            }
        }
	    */
    }

    public L2CAPConnect() {
        /*
        remoteDevices = new LinkedList();
        mutex = new Object();
        */
    }
}
/*
 * vim: et ts=4 sw=4
 */

