/*
 * (c) Copyright 2004 Mauro Tortonesi - ALL RIGHTS RESERVED.
 * 
 * This file is part of the JavaBluetooth Stack.
 * 
 * The JavaBluetooth Stack is free software; you can redistribute it 
 * and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * The JavaBluetooth Stack is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * Created on Mar 22, 2004
 * by Mauro Tortonesi
 *
 */
package org.javabluetooth.stack.hci;

/** 
 * This interface is implemented by JavaBluetooth drivers for specific HCI 
 * Transports. 
 * @see org.javabluetooth.stack.hci.BluezTransport
 * @author Mauro Tortonesi
 */
public interface HCITransport {
    /**
     * Sends an HCIPacket. The Packet bytes are send over the Transport via the
     * underlying hardware to the Bluetooth Host Controller.
     * @param packet HCI Command or Data Packet.
     * @throws HCIException
     */
    public void sendPacket(byte[] packet) throws HCIException;

    /**
     * Resets the Bluetooth Host Controller.
     * @throws HCIException
     */
    public void reset() throws HCIException;
    
    /**
     * Registers a HCITransportListener interface.
     * Only one HCITransportListener interface can be registered.
     * @throws HCIException
     */
    public void registerHCITransportListener(HCITransportListener hcitl) throws HCIException;
}
/*
 * vim: et ts=4 sw=4
 */

