#include <assert.h>
#include <errno.h>
#include <stdint.h>
#include <sys/select.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <unistd.h>

#include <bluetooth/bluetooth.h>
#include <bluetooth/hci.h>
#include <bluetooth/hci_lib.h>

#include "libBluezTransport.h"

#define PACKET_BUFFER_SIZE 4096

//#define DEBUG 1

#ifdef DEBUG
#define DPRINTF(args)	{ printf args ;  fflush(stdout); }
#define DPUTS(str)	{ puts(str); fflush(stdout); }
#else
#define DPRINTF(args)	do {} while(0);
#define DPUTS(str)	do {} while(0);
#endif

JNIEXPORT jint JNICALL Java_org_javabluetooth_stack_hci_BluezTransport_startBluez
  (JNIEnv *env, jobject obj, jobject tx_queue, jobject hcitl)
{
	int fd;
	uint8_t buf[PACKET_BUFFER_SIZE];
	struct sockaddr_hci sh;
	struct hci_filter flt;
	jclass tx_queue_cls;
	jmethodID isEmpty;

	puts("in startBluez()"); fflush(stdout);

#ifdef DEBUG
	{
		jclass tmp_cls;
		tmp_cls = (*env)->FindClass(env, "org/javabluetooth/util/PacketQueue");
		if (tmp_cls == NULL) {
			puts("tmp_cls (PacketQueue) == NULL"); fflush(stdout);
		} else {
			int is;
			
			puts("got valid tmp_cls (PacketQueue)"); fflush(stdout);
			
			is = (*env)->IsInstanceOf(env, tx_queue, tmp_cls);
			printf("tx_queue %s an instance of PacketQueue\n", 
			       (is == JNI_TRUE ? "is" : "is not"));
			fflush(stdout);
		}
		
		tmp_cls = (*env)->FindClass(env, "org/javabluetooth/stack/hci/HCIDriver");
		if (tmp_cls == NULL) {
			puts("tmp_cls (HCIDriver) == NULL"); fflush(stdout);
		} else {
			int is;
			
			puts("got valid tmp_cls (HCIDriver)"); fflush(stdout);
			
			is = (*env)->IsInstanceOf(env, hcitl, tmp_cls);
			printf("tx_queue %s an instance of HCIDriver\n", 
			       (is == JNI_TRUE ? "is" : "is not"));
			fflush(stdout);
		}
		
		tmp_cls = (*env)->FindClass(env, "org/javabluetooth/stack/hci/HCITransportListener");
		if (tmp_cls == NULL) {
			puts("tmp_cls (HCITransportListener) == NULL"); fflush(stdout);
		} else {
			int is;
			
			puts("got valid tmp_cls (HCITransportListener)"); fflush(stdout);
			
			is = (*env)->IsInstanceOf(env, hcitl, tmp_cls);
			printf("tx_queue %s an instance of HCITransportListener\n", 
			       (is == JNI_TRUE ? "is" : "is not"));
			fflush(stdout);
		}
	}
#endif
	
	/* Create HCI socket */
	fd = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI);
	if (fd < 0)
		return -1;

	//printf("socket created: %d\n", fd); fflush(stdout);

        /* Setup filter */
	hci_filter_clear(&flt);
	hci_filter_all_ptypes(&flt);
	hci_filter_all_events(&flt);
	if (setsockopt(fd, SOL_HCI, HCI_FILTER, &flt, sizeof(flt)) < 0) {
		printf("error with setsockopt(2) while setting HCI filters: %s (%d)\n", 
		       strerror(errno), errno);
		fflush(stdout);		
		return -1;
	}
	
	memset(&sh, 0, sizeof(sh));
	sh.hci_family = AF_BLUETOOTH;
	/* we use device ID 0 (hci0) for the moment */
	sh.hci_dev = 0; 
	if (bind(fd, (struct sockaddr *)&sh, sizeof(sh)) < 0) {
		printf("error with bind(2): %s (%d)\n", strerror(errno), errno);
		fflush(stdout);		
		return -1;
	}
	
	tx_queue_cls = (*env)->GetObjectClass(env, tx_queue);
	assert(tx_queue_cls != NULL);
			
	isEmpty = (*env)->GetMethodID(env, tx_queue_cls, "isEmpty", "()Z");
	assert(isEmpty != NULL);
			
	DPUTS("entering select loop");
	/* The select loop: */
	for (;;) {
		int rr, max_fd;
		fd_set read_fdset, write_fdset;
		struct timeval timeout;
		int have_packet_to_send = 0;
		jboolean tx_queue_is_empty;

		DPUTS("l0000000000000000ping");

		tx_queue_is_empty = (*env)->CallBooleanMethod(env, tx_queue, isEmpty);
		have_packet_to_send = (tx_queue_is_empty == JNI_TRUE ? 0: 1);
		//DPRINTF(("have_packet_to_send is %d:\n", have_packet_to_send));
		
		/* setup fd_set stuff for select */
		FD_ZERO(&read_fdset);
		FD_ZERO(&write_fdset);

		max_fd = fd;
		FD_SET(fd, &read_fdset);
		if (have_packet_to_send)
			FD_SET(fd, &write_fdset);

		/* setup 1 second timeout */
		timeout.tv_sec  = 1;
		timeout.tv_usec = 0;

		/* blocking select with timeout */
		rr = select(max_fd + 1, &read_fdset, &write_fdset, NULL, &timeout);

		/* handle select errors.
		 * if errno == EINTR we just retry select */
		if (rr < 0) {
			if (errno == EINTR) continue;
			return -2;
		}
		
		//printf("FD_ISSET(fd, &read_fdset) is: %d\n", FD_ISSET(fd, &read_fdset)); fflush(stdout);
		if (FD_ISSET(fd, &read_fdset)) {
			/* fd is ready for reading */
			int i;
			jbyteArray packet;
			jbyte *packet_buf;
			jclass hcitl_cls;
			jmethodID receiveData;

			/* read data from the socket */
			rr = recv(fd, buf, sizeof(buf), 0);
			
			if (rr < 0) return -3;

			/* create packet */
			packet = (*env)->NewByteArray(env, rr);
			
			packet_buf = (*env)->GetByteArrayElements(env, packet, 0);
			
			for (i = 0; i < rr; ++i) { // optimize here
				packet_buf[i] = buf[i];
			}
			
			(*env)->ReleaseByteArrayElements(env, packet, packet_buf, 0);
			
			hcitl_cls = (*env)->GetObjectClass(env, hcitl);
			assert(hcitl_cls != NULL);
			
			receiveData = (*env)->GetMethodID(env, hcitl_cls, "receiveData", "([BI)V");
			assert(receiveData != NULL);
			
			DPUTS("BluezXport: calling receiveData");
			/* send packet to the HCIPacketListener */
			(*env)->CallVoidMethod(env, hcitl, receiveData, packet, rr);
			DPUTS("BluezXport: receiveData called");
		}

		/*
		printf("have_packet_to_send is: %d\n", have_packet_to_send);
		printf("FD_ISSET(fd, &write_fdset) is: %d\n", FD_ISSET(fd, &write_fdset));
		fflush(stdout);
		*/
		if (have_packet_to_send && FD_ISSET(fd, &write_fdset)) {
			/* fd is ready for writing */
			int i;
			jclass tx_queue_cls;
			jmethodID extract;
			jbyteArray packet;
			jbyte *packet_buf;
			jsize len;

			tx_queue_cls = (*env)->FindClass(env, "org/javabluetooth/util/PacketQueue");
			//tx_queue_cls = (*env)->GetObjectClass(env, tx_queue);
			assert(tx_queue_cls != NULL);
		
			extract = (*env)->GetMethodID(env, tx_queue_cls, "extract", "()[B");
			assert(extract != NULL);
	
			/* extract packet from packet queue */
			packet = (*env)->CallObjectMethod(env, tx_queue, extract);
			/*
			if (packet == NULL) 
				puts("packet == NULL");
			else
				puts("got valid packet");
			fflush(stdout);
			*/

#if 0
			puts("23"); fflush(stdout);
			{
				jclass array_cls;
				jclass byte_cls;
				jclass byte_array_cls;
			
				array_cls = (*env)->FindClass(env, "java/lang/reflect/Array");
				if (array_cls == NULL) {
					puts("array_cls == NULL"); fflush(stdout);
				} else {
					int is;
					
					puts("got valid array_cls"); fflush(stdout);
					
					is = (*env)->IsInstanceOf(env, dummy, array_cls);
					printf("dummy %s an instance of Array\n", 
					        (is == JNI_TRUE ? "is" : "is not"));
					fflush(stdout);
				}
				
				byte_cls = (*env)->FindClass(env, "java/lang/Byte");
				if (byte_cls == NULL) {
					puts("byte_cls == NULL\n"); fflush(stdout);
				} else {
					int is;
					
					puts("got valid byte_cls"); fflush(stdout);
					
					is = (*env)->IsInstanceOf(env, dummy, byte_cls);
					printf("dummy %s an instance of Byte\n", 
					       (is == JNI_TRUE ? "is" : "is not"));
					fflush(stdout);
				}
				

				byte_array_cls = (*env)->FindClass(env, "[java/lang/Byte");
				if (byte_array_cls == NULL) {
					puts("byte_array_cls == NULL"); fflush(stdout);
				} else {
					int is;
					
					puts("got valid byte_array_cls"); fflush(stdout);
					
					is = (*env)->IsInstanceOf(env, dummy, byte_array_cls);
					printf("dummy %s an array of Byte\n", 
					       (is == JNI_TRUE ? "is" : "is not"));
					fflush(stdout);
				}
				
			}
#endif

			/* copy packet on buf */
			len = (*env)->GetArrayLength(env, packet);
			
			assert(len < PACKET_BUFFER_SIZE);
			packet_buf = (*env)->GetByteArrayElements(env, packet, 0);
			
			for (i = 0; i < len; ++i) { // optimize here
				buf[i] = packet_buf[i];
			}
			
			(*env)->ReleaseByteArrayElements(env, packet, packet_buf, JNI_ABORT);
			
			/* write data to the socket */
			//printf("len = %d\n", len);
			DPRINTF(("sending packet of length %d\n", len));
			rr = send(fd, buf, len, 0);
			if (rr < 0) {
				printf("error with send(2): %s (%d)\n", strerror(errno), errno);
				fflush(stdout);
				return -4;
			}
		}
	}

	close(fd);

	puts("exit from startBluez()"); fflush(stdout);
	
	return 0;
}

