/*
 * (c) Copyright 2004 Mauro Tortonesi - ALL RIGHTS RESERVED
 * 
 * This file is part of the JSR82ext stack.
 * 
 * The JSR82ext stack is free software; you can redistribute it 
 * and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * The JSR82ext stack is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 */

package org.javabluetooth.stack.l2cap;

import java.io.*;
import java.util.*;
import org.javabluetooth.stack.hci.HCIDriver;
import org.javabluetooth.stack.hci.HCIException;
import org.javabluetooth.stack.BluetoothStack;
import org.javabluetooth.stack.BluetoothStackLocal;
import org.javabluetooth.util.PacketQueue;

public class ASBChannel extends L2CAPChannel {
    public short psm;
    public HCIDriver hciDriver;
    private PacketQueue incomingPackets;

    public ASBChannel(short _psm) {   
        BluetoothStackLocal stack = null;
        try {
            stack = (BluetoothStackLocal)BluetoothStack.getBluetoothStack();
        }
        catch (HCIException e) {
            // TODO: handle this one
        }
        psm = _psm;
        channelState = OPEN;
        hciDriver = stack.getHCIDriver();
        incomingPackets = new PacketQueue();
    }

    public void wasDisconnected() {
        /* do nothing */
    }

    /* reads data from incoming packet queue */
    public void receive(byte[] data) throws IOException {
        /* use semaphore instead of sleep */
        while (incomingPackets.isEmpty()) {
            if (channelState == CLOSED)
                throw new IOException("L2CAP broadcast channel for psm " + 
                                      psm + " is closed.");
            try { Thread.sleep(1000); }
            catch (InterruptedException e) { }
        }
        byte[] packet = (byte[])incomingPackets.extract();
        if (packet != null) {
            int length = data.length;
            
            if (packet.length < length) 
                length = packet.length;
            
            System.arraycopy(packet, 0, data, 0, length);
            
            /* don't lose data */
            /*
            if (packet.length > length) {
                int size = packet.length - length;
                byte[] remainingData = new byte[size];
                System.arraycopy(packet, length, remainingData, 0, size);
                incomingPackets.addInFrontOfQueue(remainingData);
            }
            */
        }
        throw new InterruptedIOException();
    }
    
    /* called from L2CAPLink, puts data into incoming packet queue */
    public void receiveL2CAPPacket(byte[] data) {
        incomingPackets.insert(data);
    }

    public synchronized void close() { 
        channelState = CLOSED;
    }
    
    public synchronized void send(byte[] data) throws IOException {
        if (channelState != OPEN)
            throw new IOException("L2CAP Broadcast Channel for PSM " + 
                                  psm + " is not open.");
        sendL2CAPPacket(data);
    }
                
    public synchronized void sendL2CAPPacket(byte[] data) throws IOException {
        /* TODO: make sure that data is smaller than 64kb */
        L2CAPLink.sendL2CAPBroadcastPacket(this, data);
    }
}
/*
 * vim: et ts=4 sw=4
 */

