/*
 * (c) Copyright 2003 Christian Lorenz - ALL RIGHTS RESERVED
 * (c) Copyright 2004 Mauro Tortonesi - ALL RIGHTS RESERVED
 * 
 * This file is part of the JSR82ext stack.
 * 
 * The JSR82ext stack is free software; you can redistribute it 
 * and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * The JSR82ext stack is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 */

package org.javabluetooth.stack.l2cap;

import java.io.*;
import java.util.*;
import javax.bluetooth.L2CAPConnection;
import javax.bluetooth.RemoteDevice;
import org.javabluetooth.util.PacketQueue;
import org.javabluetooth.stack.l2cap.L2CAPLink;

/** 
 * This class is used by the Connector.open(String url) method. It encapsulates
 * the L2CAPChannel in an L2CAPConnection, which can be used by the JSR-82 
 * implementation.
 * @see	javax.bluetooth.L2CAPConnection
 * @see org.javabluetooth.stack.l2cap.L2CAPChannel
 * @author Christian Lorenz
 */
public class JSR82Channel extends L2CAPChannel implements L2CAPConnection {
    private PacketQueue incomingPackets;
    private RemoteDevice remoteDevice;

    public JSR82Channel() {
        incomingPackets = new PacketQueue();
        remoteDevice = null;
    }

    public void wasDisconnected() { }

    /** @see org.javabluetooth.stack.l2cap.L2CAPChannel#receivePacket(byte[]) */
    public synchronized void receiveL2CAPPacket(byte[] dataPacket) {
        incomingPackets.insert(dataPacket);
    }

    /** @see javax.bluetooth.L2CAPConnection#getTransmitMTU() */
    public int getTransmitMTU() throws IOException {
        /* TODO: implement getTransmitMTU */
        return 0;
    }

    /** @see javax.bluetooth.L2CAPConnection#getReceiveMTU() */
    public int getReceiveMTU() throws IOException {
        /* TODO: implement getReceiveMTU */
        return 0;
    }

    /** @see javax.bluetooth.L2CAPConnection#send(byte[]) */
    public synchronized void send(byte[] data) throws IOException {
        if (channelState == CLOSED)
            throw new IOException("L2CAP Channel is closed.");
       
        /* TODO: make sure that data is smaller than 64kb */
        sendL2CAPPacket(data);
    }

    /** @see javax.bluetooth.L2CAPConnection#receive(byte[]) */
    public synchronized int receive(byte[] inBuf) throws IOException {
        /* use semaphore instead of sleep */
        while (incomingPackets.isEmpty()) {
            if (channelState == CLOSED)
                throw new IOException("L2CAP Channel is closed.");
            try { Thread.sleep(1000); }
            catch (InterruptedException e) { }
        }
        byte[] packet = (byte[])incomingPackets.extract();
        if (packet != null) {
            int length = inBuf.length;
            if (packet.length < length) length = packet.length;
            System.arraycopy(packet, 0, inBuf, 0, length);
            return length;
        }
        throw new InterruptedIOException();
    }

    /** @see javax.bluetooth.L2CAPConnection#ready() */
    public boolean ready() throws IOException {
        return !incomingPackets.isEmpty();
    }

    /** @see javax.bluetooth.L2CAPConnection#getRemoteDevice() */
    public RemoteDevice getRemoteDevice() {
        return remoteDevice;
    }
}
/*
 * vim: et ts=4 sw=4
 */

