/**
 * Semaphore is a straightforward implementation of the well-known
 * synchronization primitive. Its counter can be initialized to any
 * nonnegative value -- by default, it is zero.
 */

package org.javabluetooth.util;

public class Semaphore {
    private int counter;
    private boolean debug;
    
    public Semaphore() {
        this(0, false);
    }
    
    public Semaphore(int i) {
        this(i, false);
    }
    
    public Semaphore(int i, boolean print_debug_info) {
        if (i < 0) throw new IllegalArgumentException(i + " < 0");
        counter = i;
        debug = print_debug_info;
        if (debug) {
            Debug.println(1, "Semaphore: created new instance with counter = " + counter);
        }         
    }
    
    /**
     * Increments internal counter, possibly awakening a thread
     * wait()ing in acquire().
     */
    public synchronized void release() {
        if (debug) {
            Debug.println(1, "Semaphore: called release() with counter = " + counter);
        }         
        if (counter == 0) {
            this.notify();
        }
        counter++;
        if (debug) {
            Debug.println(1, "Semaphore: exit from release() with counter = " + counter);
        }         
    }
    
    /**
     * Decrements internal counter, blocking if the counter is already
     * zero.
     *
     * @exception InterruptedException passed from this.wait().
     */
    public synchronized void acquire() throws InterruptedException {
        if (debug) {
            Debug.println(1, "Semaphore: called acquire() with counter = " + counter);
        }         
        while (counter == 0) {
            this.wait();
        }
        counter--;
        if (debug) {
            Debug.println(1, "Semaphore: exit from acquire() with counter = " + counter);
        }         
    }
}
/*
 * vim: et ts=4 sw=4
 */

